#!/usr/bin/env python
#
# Simple script to put uid to lower case (and avoid troubles)
#
# Copyright (C) 2011 Simone Piccardi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import getopt, re
import ldap
import time
from os import *
from sys import *

# help on line
def usage():
    print "Usage: lowercaseuid.py [OPTIONS]"
    print "  -h        print this help"
    print "  -d        debug run, just print commands and values"
    print "  -u usr    use ou=usr as base search for user (instead of Users)"
    print "  -p pwd    LDAP access pass (instead of trying read ldap.secret)"
    print
    print "ARGUMENTS:"
    print "  none"
    print " "

# option parsing
try:
    opts, args = getopt.getopt(argv[1:], 'hdp:u:')
    optval={}
    for opt, val in opts:
        if opt == "-h":
            usage()
            exit(0)
        else:
            optval[opt]=val
except getopt.GetoptError:
    usage()
    exit(2)

# check arguments: none must be present
if len(args) != 0:
    print "No arguments for this script"
    print
    usage()
    exit(1)

# Get LDAP parameters from standard files
flags  = re.IGNORECASE
try:
    regexp = r'base\s*(.*)'
    regx   = re.compile(regexp, flags)
    conf   = file("/etc/ldap/ldap.conf").read()
    base   = regx.findall(conf)[0]
    regexp = r'uri\s*(.*)'
    regx   = re.compile(regexp, flags)
    uri    = regx.findall(conf)[0]
    if optval.has_key('-d'):
        print base
        print uri
except Exception,e:
    print "cannot read configuration from /etc/ldap/ldap.conf"
    print e

if  optval.has_key('-p'):
    password = optval['-p']
else:
    try:
        regexp = r'masterPw="(.*)"'
        regx   = re.compile(regexp, flags)
        conf   = file("/etc/smbldap-tools/smbldap_bind.conf").read()
        password = regx.findall(conf)[0]
    except Exception,e:
        print "cannot read password from /etc/smbldap-tools/smbldap_bind.conf"
        print e


# LDAP connection
try: 
    l = ldap.initialize(uri)
    l.protocol_version = ldap.VERSION3
    username = "cn=admin,"+base
    l.simple_bind(username, password)

except ldap.LDAPError, e:
    print "Connection error", e

# LDAP search parameters
if optval.has_key('-u'):
    baseDN = "ou="+optval['-u']+","+base
else:
    baseDN = "ou=Users,"+base

if optval.has_key('-d'):
    print baseDN

searchScope = ldap.SCOPE_ONELEVEL
searchFilter = "uid=*"
retrieveAttributes = ["uid"]

# do LDAP search and set quota on results
try:
    r = l.search_s(baseDN, searchScope, searchFilter, retrieveAttributes)
    if r != []:
        for dn, entry in r:
            rdn="uid="+entry['uid'][0].lower()+","+baseDN
            if optval.has_key('-d'):
                print entry
                print dn
                print rdn
            else:
                l.rename_s(dn,rdn)
    else:
        print "No Results from LDAP"
        exit(1)

except ldap.LDAPError, e:
    print "Search error", e
    
